@@ -1,13 +1,18 @@
 """Line strings and related utilities
 """
 
+import sys
+
+if sys.version_info[0] < 3:
+    range = xrange
+
 from ctypes import c_double, cast, POINTER
-from ctypes import ArgumentError
 
 from shapely.coords import required
 from shapely.geos import lgeos, TopologicalError
 from shapely.geometry.base import BaseGeometry, geom_factory, JOIN_STYLE
 from shapely.geometry.proxy import CachingGeometryProxy
+from shapely.geometry.point import Point
 
 __all__ = ['LineString', 'asLineString']
 
@@ -26,8 +31,8 @@ class LineString(BaseGeometry):
         ----------
         coordinates : sequence
             A sequence of (x, y [,z]) numeric coordinate pairs or triples or
-            an object that provides the numpy array interface, including another
-            instance of LineString.
+            an object that provides the numpy array interface, including
+            another instance of LineString.
 
         Example
         -------
@@ -82,8 +87,8 @@ class LineString(BaseGeometry):
         return self.coords.xy
 
     def parallel_offset(
-        self, distance, side,
-        resolution=16, join_style=JOIN_STYLE.round, mitre_limit=1.0):
+            self, distance, side,
+            resolution=16, join_style=JOIN_STYLE.round, mitre_limit=1.0):
 
         """Returns a LineString or MultiLineString geometry at a distance from
         the object on its right or its left side.
@@ -107,7 +112,7 @@ class LineString(BaseGeometry):
         try:
             return geom_factory(self.impl['parallel_offset'](
                 self, distance, resolution, join_style, mitre_limit,
-                bool(side=='left')))
+                bool(side == 'left')))
         except WindowsError:
             raise TopologicalError()
 
@@ -183,13 +188,13 @@ def geos_linestring_from_py(ob, update_geom=None, update_ndim=0):
             cs = lgeos.GEOSGeom_getCoordSeq(update_geom)
             if n != update_ndim:
                 raise ValueError(
-                "Wrong coordinate dimensions; this geometry has dimensions: %d" \
-                % update_ndim)
+                    "Wrong coordinate dimensions; this geometry has "
+                    "dimensions: %d" % update_ndim)
         else:
             cs = lgeos.GEOSCoordSeq_create(m, n)
 
         # add to coordinate sequence
-        for i in xrange(m):
+        for i in range(m):
             dx = c_double(cp[n*i])
             dy = c_double(cp[n*i+1])
             dz = None
@@ -208,12 +213,24 @@ def geos_linestring_from_py(ob, update_geom=None, update_ndim=0):
 
     except AttributeError:
         # Fall back on list
-        m = len(ob)
+        try:
+            m = len(ob)
+        except TypeError:  # Iterators, e.g. Python 3 zip
+            ob = list(ob)
+            m = len(ob)
+
         if m < 2:
             raise ValueError(
                 "LineStrings must have at least 2 coordinate tuples")
+
+        def _coords(o):
+            if isinstance(o, Point):
+                return o.coords[0]
+            else:
+                return o
+
         try:
-            n = len(ob[0])
+            n = len(_coords(ob[0]))
         except TypeError:
             raise ValueError(
                 "Input %s is the wrong shape for a LineString" % str(ob))
@@ -224,14 +241,14 @@ def geos_linestring_from_py(ob, update_geom=None, update_ndim=0):
             cs = lgeos.GEOSGeom_getCoordSeq(update_geom)
             if n != update_ndim:
                 raise ValueError(
-                "Wrong coordinate dimensions; this geometry has dimensions: %d" \
-                % update_ndim)
+                    "Wrong coordinate dimensions; this geometry has "
+                    "dimensions: %d" % update_ndim)
         else:
             cs = lgeos.GEOSCoordSeq_create(m, n)
 
         # add to coordinate sequence
-        for i in xrange(m):
-            coords = ob[i]
+        for i in range(m):
+            coords = _coords(ob[i])
             # Because of a bug in the GEOS C API,
             # always set X before Y
             lgeos.GEOSCoordSeq_setX(cs, i, coords[0])
@@ -247,9 +264,11 @@ def geos_linestring_from_py(ob, update_geom=None, update_ndim=0):
     else:
         return lgeos.GEOSGeom_createLineString(cs), n
 
+
 def update_linestring_from_py(geom, ob):
     geos_linestring_from_py(ob, geom._geom, geom._ndim)
 
+
 # Test runner
 def _test():
     import doctest
